
#include <bobpp>
#include <string>
#include <vector>
#include <signal.h>


namespace Bob {

#ifdef ENABLE_DEBUG
bool debug_config::mem=false;
bool debug_config::user=false;
bool debug_config::algo=false;
bool debug_config::env=false;

void debug_config::opt() {
      core::opt().add(std::string("--dbg"), Property("-mem", "print information on memory managment",&debug_config::mem));
      core::opt().add(std::string("--dbg"), Property("-al", "print information on algorithm execution",&debug_config::algo));
      core::opt().add(std::string("--dbg"), Property("-user", "print information generate by the user side",&debug_config::user));
      core::opt().add(std::string("--dbg"), Property("-env", "print information generate by the environment side",&debug_config::env));
}
#else
void debug_config::opt() { }

#endif


std::string core::SerialFileName;
string core::logfile = "file.log";
bool core::verb = false;
Opt core::theopt;
Log core::thelog;
long core::reft;
double core::reftd;
int core::btdthreshold;
long core::nid=0;
bool core::dispbestval=false;

/*------------------ Core methods ----------------------*/

void core::Config(int n, char **v) {

  opt().add(std::string("--"), Property("-help", "Display the help"));
  opt().add(std::string("--"), Property("-t", "Depth threshold for BestThenDepth priority", 10,&btdthreshold));
  opt().add(std::string("--"), Property("-lf", "log to file",&verb));
  opt().add(std::string("--"), Property("-of", "log output file", "file.log",&logfile));
  opt().add(std::string("--"), Property("-s", "display current best solution value",&dispbestval));
#ifndef WIN32
  opt().add(std::string("--"), Property("-ln", "log to network host"));
  opt().add(std::string("--"), Property("-on", "log network host, format = address[:port]", "localhost:2222"));
#endif
  debug_config::opt();
  opt().parse(n, v);
#ifndef WIN32
  verb = verb || opt().BVal("--", "-ln")
#endif
  ;
  reft = Time();
  reftd = dTime();

  //signal(SIGQUIT,StopandSave);
  //signal(SIGINT,AffStats);
  if (opt().BVal("--", "-help")) {
    opt().Usage();
    exit(0);
  }
  if (isVerb()) {
#ifndef WIN32
    if (opt().BVal("--", "-ln") ) {
      std::string host = opt().SVal("--", "-on");
      string::size_type pos = host.rfind(":", host.size());
      int port;
      char s[256];
      if (pos == string::npos)
        port = 2222;
      else {
        std::string port_str = host.substr(pos + 1);
        std::istringstream iss(port_str);
        iss >> std::dec >> port;
        host = host.substr(0, pos);
      }
      log().setLog(new Log_net(host, port));
      sprintf(s, "net://%s:%d", host.c_str(), port);
      logfile = s;
    } else
#endif
      log().setLog(new Log_file(logfile));
    std::cout << " Bob++ Version 0.1 c BLC 2006" << std::endl;
    std::cout << " Verbose Mode on " << std::endl;
    std::cout << " LogFile :" << logfile << std::endl;
  }
}

/// Get a BobSerialize generally this is the File version
/*
std::string core::SerialFileName() {
	ostringstream os;
	os << "bobpp_"<<getpid()<<s;
	return os.rdbuf()->str();
}*/

void core::End() {
  //std::cout << "That's all folks !" << std::endl;
  opt().clear();
}

long core::Time(long b) {
  struct timeval t;
  gettimeofday(&t, NULL);
  return (t.tv_sec) - (reft + b);
}
double core::dTime(double b) {
  struct timeval t;
  gettimeofday(&t, NULL);
  double d = t.tv_sec + 1e-6 * t.tv_usec;
  return d -(reftd + b);
}
string core::sTime(long b) {
  char buf[120];
  sprintf(buf, "%02ldh:%02ldm:%02lds", b / (3600), (b / 60) % 60, b % 60);
  return string(buf);
}

long core::get_new_id(long v) {
   long _nid;
   log().lock();
   _nid = nid++;
   log().unlock();
   return _nid;
}

};
