
/* 
 * Bob++/include/bob_core.h
 * 
 * This source include the abstract data type for the application
 * core. 
 */

#ifndef BOBCORE
#define BOBCORE

#include <sys/time.h>
#include <time.h>

namespace Bob {

/*------------------ core Class-------------*/
/** @defgroup CoreClassGRP The Bob::core Class
 *  The core class contains static datas and methods, that are globally
 *  used by each of the Bob++ components. It stores
 *  -# the options objects that stores the properties (@ref OptGRP)
 *  -# some constants or value that are associated with options
 *  -# the Log object (see @ref IOGRP) used to log the activity of the different algorithms
 *  -# methods to get time as long, double, or string types.
 *
 *  @brief the core class stores many global features
 *  @ingroup CoreGRP 
  * @{
  *
*/

/** the Core class
 */
class core {
private:
   static std::string SerialFileName;
protected: 
   /// The file log for the execution
   static std::string logfile;
   /// Verbose level
   static bool verb;		
   /// The Option
   static Opt theopt;
   /// Interface for log information.
   static Log thelog;
   /// the reference time.
   static long reft;
   /// the reference time with microsecond precision.
   static double reftd;
   // OStream for log information.
   //static SerialFile sout;
   // OStream for log information.
   //static DeserialFile sin;
   /// Depth threshold for BestThenDepth priority
   static int btdthreshold;
   /// the node identifier
   static long nid;
   /// display the value of the current best solution
   static bool dispbestval;
public:
   /// constructor
   core() {}
   /// Destructor
   ~core() {}
   /// Parse the internal option of Bob++
   static void Config(int n, char **v);
   /// Gets the verbose level
   static int isVerb() { return verb; }
   /// method to get the option
   static int BTDthreshold() { return btdthreshold; }
   /// if we need to display the best value at each goal update
   static bool dispBestVal() { return dispbestval; }
   /// method to get the option
   static Opt &opt() { return theopt; }
   /// method to get the logging object.
   static Log &log() { return thelog; }
   /// End the core (not really used)
   static void End();
   /// static method to get the time in seconds
   static long Time(long b=0);
   /// static method to get the time in seconds with microsecond precision
   static double dTime(double b=0.0);
   /// static method to get the time in a human redable way
   static std::string sTime(long b);
   /// static method to get a node identifier
   static long get_new_id(long v=0);
};


/** RegistStatException. An object of this class is throwed 
 * there is error in the Bob++ classes.
*/
class RegistStatException: public Exception {
public:
   /** Default constructor
        * @param msg is the message of the execption
   */
  RegistStatException(const std::string& msg )
    : Exception("Register Statistics Exception :",msg) {}
};


/**
 *  @}
 */
}
#endif

