#include <unistd.h>
#include <bobpp>
#include <string>

#include <sys/types.h>
#ifndef WIN32
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#endif

namespace Bob {

void Log_file::out(strbuff<> const &lm){
           char s[16];
           sprintf(s,"%.06f ",core::dTime());
           os << s << lm;
}

#ifndef WIN32
Log_net::Log_net(std::string host, int port):Log_base(),os(-1)
{
	struct sockaddr_in adr;
  struct hostent *hp;

	hp = gethostbyname( host.c_str() );
  if( hp == NULL )
  	herror( "Log_net::Log_net" ), exit(1);
  os = socket(PF_INET, SOCK_DGRAM, 0);
  if( os == -1 )
  	perror( "Log_net::Log_net" ), exit(1);
	adr.sin_family = AF_INET;
  adr.sin_port = htons( port );
  memcpy( &adr.sin_addr, hp->h_addr, hp->h_length );
  if( connect( os, (struct sockaddr*)&adr, sizeof(adr) ) == -1 )
  	perror( "Log_net::Log_net" ), exit(1);
}


Log_net::~Log_net() {
  if( close(os) )
  	perror( "Log_net::~Log_net" ), exit(1);
}

/// display the message lm to the log
void Log_net::out(strbuff<> const &lm) {
         send(os, lm.str(), lm.size(), 0);
}
#endif

Log::Log():l(new Log_base()),ei(new SeqId()) {pthread_mutex_init(&mut,0); };

Log::~Log() {delete l;pthread_mutex_destroy(&mut);delete ei; }

/// Set the log object
void Log::setLog(Log_base *lb) { 
           if ( l!=0 ) delete l;
           l=lb; 
}
/// Set the log object
void Log::setEId(EnvId *_ei) { 
           if ( ei!=0 ) delete ei;
           ei=_ei; 
}

/// display the id
void Log::out_nolock(strbuff<> const &lm) {
         l->out(lm);
}

/// display the id
void Log::out(bool eb,strbuff<> const &lm) {
         strbuff<> tmp;
         if ( eb ) tmp << ei->id() <<":";
         tmp<< lm ;
         out(tmp);
}

/// display the id
void Log::out(strbuff<> const &lm) {
         lock();
         l->out(lm);
         unlock();
}
/// display the id
void Log::out(const char *s,char c,int n,bool g) {
         strbuff<> st;
         if ( g ) {
            st << "*:"<<s <<" "<<c<<" "<<n<<"\n";
         } else {
            st << ei->id()<<":"<<s <<" "<<c<<" "<<n<<"\n";
         }
         out(st);
}

};
