
#include <bobpp>
#include <signal.h>
#include <iomanip>
#include <iostream>
#include <sstream>


namespace Bob {

void stat_time::start() {
  dt = 0L;
  nbr++;
  st = core::dTime();
  core::log().out(str(), 'S', nbr, global);
}

/// Set the end time
void stat_time::end() {
  dt = core::dTime(st);
  cumt += dt;
  core::log().out(str(), 'E', nbr, global);
}

void stat_counter::update_log() {
  update_mix();
  core::log().out(str(), ' ', nb,global);
}

void stat_counter::update_mix() {
  if (nb > max) max = nb;
}

void stat::merge(const stat &s) {
  DBGAFF_ENV("stat::merge", "merge all the stats");
  for (MapCount::const_iterator i = s.mc.begin(); i != s.mc.end() ; i++) {
    mc[i->first].merge(i->second);
  }
  for (MapTime::const_iterator i = s.mt.begin(); i != s.mt.end() ; i++) {
    mt[i->first].merge(i->second);
  }
  for (MapStat::const_iterator i = s.mss.begin(); i != s.mss.end() ; i++) {
    if (i->second != 0) {
      mss[i->first]->merge(*(i->second));
    }
  }
}

#define COUN_SP 6
#define TIME_SP 10

/* Dispaly of stat counters */
void formatL(std::stringstream &os, const stat_counter &st) {
  os << std::setw(COUN_SP) << st.id() << " ";
}

void formatD(std::stringstream &os, const stat_counter &st) {
  os << std::setw(COUN_SP) << st.get() << " ";
}

void formatL(std::stringstream &os, const MapCount &mc) {
  for (MapCount::const_iterator i = mc.begin(); i != mc.end() ; i++) {
    formatL(os, i->second);
  }
}
void formatD(std::stringstream &os, const MapCount &mc) {
  for (MapCount::const_iterator i = mc.begin(); i != mc.end() ; i++) {
    formatD(os, i->second);
  }
}

/* Dispaly of stat timers */
void formatL(std::stringstream &os, const stat_time &st) {
  os << std::setw(TIME_SP) << st.id() << " ";
}

void formatD(std::stringstream &os, const stat_time &st) {
  os << std::fixed << std::setprecision(0) << std::setw(3) << st.getn()
  << ":" << std::setprecision(4) << st.getc() << " ";
}

void formatL(std::stringstream &os, const MapTime &mt) {
  for (MapTime::const_iterator i = mt.begin(); i != mt.end() ; i++) {
    formatL(os, i->second);
  }
}
void formatD(std::stringstream &os, const MapTime &mt) {
  for (MapTime::const_iterator i = mt.begin(); i != mt.end() ; i++) {
    formatD(os, i->second);
  }
}

void stat::update() { 
  for (MapCount::iterator i = mc.begin(); i != mc.end() ; i++) {
    i->second.update();
  }
  for (MapTime::iterator i = mt.begin(); i != mt.end() ; i++) {
    i->second.update();
  }
  for (MapStat::iterator i = mss.begin(); i != mss.end() ; i++) {
    if (i->second!=0 ) {
         i->second->update();
    }
  }

}

void stat::name(buff_st &os) const { 
  for (MapCount::const_iterator i = mc.begin(); i != mc.end() ; i++) {
    i->second.name(os);
  }
  for (MapTime::const_iterator i = mt.begin(); i != mt.end() ; i++) {
    i->second.name(os);
  }
  for (MapStat::const_iterator i = mss.begin(); i != mss.end() ; i++) {
    if (i->second!=0 ) {
         i->second->name(os);
    }
  }

}

void stat::namelog() const {
   buff_st bs;
   bs <<"\n{\n";
   name(bs);
   bs <<"}\n";
   core::log().out(bs);
}

void stat::PrtL(ostream &os) {
  std::stringstream ss(stringstream::in | stringstream::out);
  DBGAFF_ENV("stat::merge", "merge all the stats");
  ss << "-------Stats for : " << id() << std::endl;
  formatL(ss, (const MapTime &)mt);
  formatL(ss, (const MapCount &)mc);
  ss << std::endl;
  os << ss.str();
}

void stat::PrtD(ostream &os) {
  std::stringstream ss(stringstream::in | stringstream::out);
  DBGAFF_ENV("stat::merge", "merge all the stats");
  formatD(ss, (const MapTime &)mt);
  formatD(ss, (const MapCount &)mc);
  ss << std::endl;
  os << ss.str();
}


void stat::Pack(Serialize &bs) const {
  int nb;
  int j;
  DBGAFF_ENV("stat::Pack", "Packing stat");
  nb = mc.size();
  bs.Pack(&nb, 1);
  for (MapCount::const_iterator i = mc.begin(); i != mc.end() ; i++) {
    bs.Pack(&i->first, 1);
    i->second.Pack(bs);
  }
  nb = mt.size();
  bs.Pack(&nb, 1);
  for (MapTime::const_iterator i = mt.begin(); i != mt.end() ; i++) {
    bs.Pack(&i->first, 1);
    i->second.Pack(bs);
  }
  nb = mss.size();
  j = 0;
  for (MapStat::const_iterator i = mss.begin(); i != mss.end() ; i++) {
    if (i->second != 0) {
      j++;
    }
  }
  bs.Pack(&j, 1);
  for (MapStat::const_iterator i = mss.begin(); i != mss.end() ; i++) {
    if (i->second != 0) {
      bs.Pack(&i->first, 1);
      i->second->Pack(bs);
    }
  }
}

void stat::UnPack(DeSerialize &bs)  {
  DBGAFF_ENV("stat::UnPack", "UnPacking stat");
  char s;
  stat_counter sc;
  stat_time st;
  stat *sta;
  int i, nb;
  bs.UnPack(&nb, 1);
  for (i = 0; i < nb ; i++) {
    bs.UnPack(&s, 1);
    sc.UnPack(bs);
    mc[s] = sc;
  }
  bs.UnPack(&nb, 1);
  for (i = 0; i < nb ; i++) {
    bs.UnPack(&s, 1);
    st.UnPack(bs);
    mt[s] = st;
  }
  bs.UnPack(&nb, 1);
  for (i = 0; i < nb ; i++) {
    bs.UnPack(&s, 1);
    sta = new stat();
    sta->UnPack(bs);
    mss[s] = sta;
  }
}


};
